use strict;
use warnings;
sub Genmark_frame_calculate
{
	my $genmark_orf_list=shift;
	chomp $genmark_orf_list;
	open GM,$genmark_orf_list or die "Cannot open GeneMark orf list file\n";
	my @frame_calculated_data=();
	my $genome_length=shift;
	chomp $genome_length;
	while(my $line=<GM>)
	{
		chomp $line;
		if($line=~/^\s+\d/)
		{
			$line=~s/\r//g;
			my @line_data=split(/\s+/,$line);			
			if($line_data[1]>0)
			{
				my $strand=$line_data[2];
				my $start=$line_data[3];
				$start=~s/\s|\r|\n//g;
				$start=~s/[^0-9]//g;
				my $end=$line_data[4];
				$end=~s/\s|\r|\n//g;
				$end=~s/[^0-9]//g;
				my $strand_n_frame='';
				if($strand eq '+')
				{
					my $frame=0;
					my $remain=($start-1)%3;
					$frame=$remain+1;
					$strand_n_frame='+'.$frame;
				}
				if($strand eq '-')
				{
					my $frame=0;
					my $remain=($genome_length-$end)%3;
					$frame=$remain+1;
					$strand_n_frame='-'.$frame;
				}
			  	my $new_line=$line_data[1]."\t".$start."\t".$end."\t".$strand;
				push(@frame_calculated_data,$new_line."\t".$strand_n_frame);				
			}
		}
	}
	close GM;
	return(@frame_calculated_data);
}
sub Glimmer_frame_calculate
{
	my $glimmer_orf_file=shift;
	chomp $glimmer_orf_file;
	open GL,$glimmer_orf_file or die "Cannot open GeneMark orf list file\n";
	my @frame_calculated_data=();
	my $genome_length=shift;
	chomp $genome_length;
	while(my $line=<GL>)
	{
		chomp $line;
		if($line!~/^>/)
		{
			$line=~s/\r//g;
			my @line_data=split(/\s+/,$line);			
			if($line_data[1]>0)
			{

				my $strand='+';
				if($line_data[3]<0)
				{
					$strand='-';
				}
				my $start=$line_data[1];
				$start=~s/\s|\r|\n//g;
				$start=~s/[^0-9]//g;
				my $end=$line_data[2];
				$end=~s/\s|\r|\n//g;
				$end=~s/[^0-9]//g;
				my $strand_n_frame='';
				my $new_line='';
				if($strand eq '+')
				{
					my $frame=0;
					my $remain=($start-1)%3;
					$frame=$remain+1;
					$strand_n_frame='+'.$frame;
					$new_line=$line_data[0]."\t".$start."\t".$end."\t".$strand;
				}
				if($strand eq '-')
				{
					my $frame=0;
					my $remain=($genome_length-$start)%3;
					$frame=$remain+1;
					$strand_n_frame='-'.$frame;
					$new_line=$line_data[0]."\t".$end."\t".$start."\t".$strand;
				}
			  	
				push(@frame_calculated_data,$new_line."\t".$strand_n_frame);				
			}
		}
	}
	close GL;
	return(@frame_calculated_data);
}
sub FGENESB_frame_calculate
{
	my $fgenesb_outfile=shift;
	chomp $fgenesb_outfile;
	open FGEN,$fgenesb_outfile or die "Cannot open GeneMark orf list file\n";
	my @frame_calculated_data=();
	my $genome_length=shift;
	chomp $genome_length;
	while(my $line=<FGEN>)
	{
		chomp $line;
		if($line=~/^\s+\d/)
		{
			$line=~s/\r//g;
			my @line_data=split(/\s+/,$line);			
			if($line_data[7]=~/CDS/)
			{
				
				my $strand=$line_data[6];
				my $start=$line_data[8];
				$start=~s/\s|\r|\n//g;
				$start=~s/[^0-9]//g;
				my $end=$line_data[10];
				$end=~s/\s|\r|\n//g;
				$end=~s/[^0-9]//g;
				my $strand_n_frame='';
				if($strand eq '+')
				{
					my $frame=0;
					my $remain=($start-1)%3;
					$frame=$remain+1;
					$strand_n_frame='+'.$frame;
				}
				if($strand eq '-')
				{
					my $frame=0;
					my $remain=($genome_length-$end)%3;
					$frame=$remain+1;
					$strand_n_frame='-'.$frame;
				}
			  	my $new_line=$line_data[1]."\t".$start."\t".$end."\t".$strand;
				push(@frame_calculated_data,$new_line."\t".$strand_n_frame);				
			}
		}
	}
	close FGEN;
	return(@frame_calculated_data);
}
sub prodigal_frame_calculate
{
	my $prodigal_orf_file=shift;
	chomp $prodigal_orf_file;
	open PROD,$prodigal_orf_file or die "Cannot open GeneMark orf list file\n";
	my @frame_calculated_data=();
	my $genome_length=shift;
	chomp $genome_length;
	
	while(my $line=<PROD>)
	{
		chomp $line;
		if($line=~/^\s+CDS/)
		{
			
			$line=~s/\r//g;
			my @line_data=split(/\s+/,$line);			
			my $strand='+';
			my $start_end=$line_data[2];
			if($start_end=~/complement/)
			{
				$strand='-';
				$start_end=~s/complement|\(|\)//i;
			}
			my @start_n_end=split(/\.\./,$start_end);
			my $start=$start_n_end[0];
			my $end=$start_n_end[1];
				
				$start=~s/\s|\r|\n//g;
				$start=~s/[^0-9]//g;
				
				$end=~s/\s|\r|\n//g;
				$end=~s/[^0-9]//g;
				my $strand_n_frame='';
				if($strand eq '+')
				{
					my $frame=0;
					my $remain=($start-1)%3;
					$frame=$remain+1;
					$strand_n_frame='+'.$frame;
				}
				if($strand eq '-')
				{
					my $frame=0;
					my $remain=($genome_length-$end)%3;
					$frame=$remain+1;
					$strand_n_frame='-'.$frame;
				}
			  	my $new_line=$line_data[1]."\t".$start."\t".$end."\t".$strand;
				push(@frame_calculated_data,$new_line."\t".$strand_n_frame);				
			}
		}
	close PROD;
	return(@frame_calculated_data);
}

sub GTF_parser
{
	my $gtf_file=shift;
	chomp $gtf_file;
	open(IN,$gtf_file) or die("Can not open GTF file $gtf_file");
	my @exon_data=();
	while (my $line=<IN>)
	{
		chomp $line;
		if($line=~/^\#/)
			{
				next;
			}
		my @line_data=split(/\t/,$line);
		if(scalar(@line_data)<9)
			{
				next;	
			}
		#if($line_data[3]=~/[^\d]/)
		#	{
		#		next;
		#	}
		my $desc=$line_data[8];
		my @desc_vals=split(/\;/,$desc);
		my $gene_id=$desc_vals[0];
		if($line_data[2] eq 'CDS')
			{
				push(@exon_data,[$gene_id,$line_data[3],$line_data[4],$line_data[6]]);
			}
	}
	close IN;
	return(\@exon_data);
}

sub Genbank_parser
{
	use Bio::SeqIO;
	use Bio::SeqIO::genbank;
	use Bio::Seq::RichSeq;
	use Bio::DB::Taxonomy::list;

	my $genbank_file=shift;
 	chomp $genbank_file;
	my $seqio_object = Bio::SeqIO->new(-file => "$genbank_file" );       
	my $seq_object=$seqio_object->next_seq;	
	my $gene_id='';
	my @positions=();
	my $seq_start=0;
	my $seq_end=0;
	my $strand='+';
	for my $feature ($seq_object->get_SeqFeatures) 
		{
			if($feature->primary_tag eq 'source')
			{
				for my $location ( $feature->location)
				{
					$seq_start=$location->start;
					$seq_end= $location->end ;
				}
			}
			if($feature->primary_tag eq 'gene')
			{
				for my $location ( $feature->location)
				{			
					if($location->strand == -1)
						{
							$strand='-';
						}
					else
						{
							$strand='+';
						}
					#push(@positions,['GENE',$location->start,$location->end,$strand]);
				}
			}
		   if ( $feature->location->isa('Bio::Location::SplitLocationI')&& $feature->primary_tag eq 'CDS' ) 
			   {
								 # print "CDS\t",$feature->location->start,"\t",$feature->location->end,"\n";
					my @locus_ids=$feature->get_tag_values('protein_id');
					my $locus=$locus_ids[0];
				  if($feature->location->strand == -1)
						{
							$strand='-';
						}
						else
						{
							$strand='+';
						}
				  #push(@positions,["CDS",$feature->location->start,$feature->location->end,$strand]);
				  for my $location ( $feature->location->sub_Location )
					  {					 	
						if($location->strand == -1)
						{
							$strand='-';
						}
						else
						{
							$strand='+';
						}
						push(@positions,[$locus,$location->start,$location->end,$strand]);				
			    		  }
			  }
			elsif($feature->primary_tag eq 'CDS')
			{
				
				my @locus_ids=$feature->get_tag_values('locus_tag');
					my $locus=$locus_ids[0];	
				if($feature->location->strand == -1)
						{
							$strand='-';
						}
						else
						{
							$strand='+';
						}
				push(@positions,[$locus,$feature->location->start,$feature->location->end,$strand]);
				#print "Strand\t",$feature->location->strand,"\n";
				#print "CDS\t",$feature->location->start,"\t",$feature->location->end,"\n";
			}
			else
			{
				next;
			}		
		}
	return($seq_end,\@positions);
}


1;
